// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------



// transforms
float4x4 tW: WORLD;       
float4x4 tV: VIEW;        
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

float defaultdistance = 1;
float4 color : COLOR = 0.5;

float tresh <string uiname="treshold";> =4.;
float offset   <string uiname="offset";> =5.;

int numBlobs = 2;

//texture
texture posTex <string uiname="Position Texture";>;
sampler posSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (posTex);          //apply a texture to the sampler
    MipFilter = POINT;         //sampler states
    MinFilter = POINT;
    MagFilter = POINT;
};

texture colorTex <string uiname="Color Texture";>;
sampler colorSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (colorTex);          //apply a texture to the sampler
    MipFilter = POINT;         //sampler states
    MinFilter = POINT;
    MagFilter = POINT;
};



struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS( float4 PosO  : POSITION, float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //texturecoordinates
    Out.TexCd = TexCd;

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
float4 psColorTexture(vs2ps In): COLOR
{
    float d = 0.;
    float sum = 0.;

    float4 c=0.;

    for(int i=0; i<numBlobs; i++)
    {
         float4 b = tex2D(posSamp, float2((float)i/(float)numBlobs, 0.));
         d=distance(In.TexCd, b.xy) / defaultdistance;
         sum+=b.z/(d*d);

         float4 cc =  tex2D(posSamp, float2((float)i/(float)numBlobs, 0.));
         c += (cc/(d+d));
    }

    float col=(sum/tresh )-offset;
    return (col/1.) * (c/numBlobs);
}

float4 psOneColor(vs2ps In): COLOR
{
    float d = 0.;
    float sum = 0.;

    for(int i=0; i<numBlobs; i++)
    {
         float4 b = tex2D(posSamp, float2((float)i/(float)numBlobs, 0.));
         d=distance(In.TexCd, b.xy) / defaultdistance;
         sum+=b.z/(d*d);
    }

    float col=(sum/tresh )-offset;
    return col * color;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique metaBall2DColorTexture
{
    pass P0
    {
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 psColorTexture();
    }
}

technique metaBall2DOneColor
{
    pass P0
    {
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 psOneColor();
    }
}
